@@ -37,9 +37,10 @@ def md5sum(path):
     return result
 
 
-def check_modified(file_path, state):
+def check_modified(file_path, file_stat, state):
     """check if file in file_path is modified from previous "state"
     @param file_path (string): file path
+    @param file_stat: the value returned from os.stat(file_path)
     @param state (tuple), timestamp, size, md5
     @returns (bool):
     """
@@ -48,10 +49,10 @@ def check_modified(file_path, state):
 
     timestamp, size, file_md5 = state
     # 1 - if timestamp is not modified file is the same
-    if USE_FILE_TIMESTAMP and os.path.getmtime(file_path) == timestamp:
+    if USE_FILE_TIMESTAMP and file_stat.st_mtime == timestamp:
         return False
     # 2 - if size is different file is modified
-    if os.path.getsize(file_path) != size:
+    if file_stat.st_size != size:
         return True
     # 3 - check md5
     return file_md5 != md5sum(file_path)
@@ -184,18 +185,21 @@ class DbmDB(object):
         # return key in self._dbm
         return key.encode('utf-8') in self._dbm
 
+
     def get(self, task_id, dependency):
         """Get value stored in the DB.
 
         @return: (string) or (None) if entry not found
         """
-        # optimization. check this after reduce call count
+        # optimization, just try to get it without checking it exists
         if task_id in self._db:
             return self._db[task_id].get(dependency, None)
-
-        if task_id not in self._db and self._in_dbm(task_id):
-            self._db[task_id] = json.loads(self._dbm[task_id].decode('utf-8'))
-        if task_id in self._db:
+        else:
+            try:
+                task_data = self._dbm[task_id]
+            except KeyError:
+                return
+            self._db[task_id] = json.loads(task_data.decode('utf-8'))
             return self._db[task_id].get(dependency, None)
 
 
@@ -366,9 +370,11 @@ class DependencyBase(object):
         changed = []
         status = 'up-to-date' # initial assumption
         for dep in tuple(task.file_dep):
-            if not os.path.exists(dep):
+            try:
+                file_stat = os.stat(dep)
+            except os.error:
                 raise Exception("Dependent file '%s' does not exist." % dep)
-            if check_modified(dep, self._get(task.name, dep)):
+            if check_modified(dep, file_stat, self._get(task.name, dep)):
                 changed.append(dep)
                 status = 'run'
 
