@@ -32,7 +32,8 @@ class ConsoleReporter(object):
     def execute_task(self, task):
         """called when excution starts"""
         # ignore tasks that do not define actions
-        if task.actions:
+        # ignore private/hidden tasks (tasks that start with an underscore)
+        if task.actions and (task.name[0] != '_'):
             self.write('.  %s\n' % task.title())
 
     def add_failure(self, task, exception):
@@ -76,10 +77,10 @@ class ConsoleReporter(object):
             task = result['task']
             if self.show_out:
                 out = "".join([a.out for a in task.actions if a.out])
-                self.write("%s\n" % out)
+                self.write("%s\n" % out.encode('utf-8'))
             if self.show_err:
                 err = "".join([a.err for a in task.actions if a.err])
-                self.write("%s\n" % err)
+                self.write("%s\n" % err.encode('utf-8'))
 
         if self.runtime_errors:
             self.write("#"*40 + "\n")
