@@ -21,17 +21,17 @@ PROGRAM = "python %s/sample_process.py" % TEST_PATH
 class TestTaskCheckInput(object):
 
     def testOkType(self):
-        task.Task.check_attr_input('xxx', 'attr', [], ([int, list],[]))
+        task.Task.check_attr('xxx', 'attr', [], ([int, list],[]))
 
     def testOkValue(self):
-        task.Task.check_attr_input('xxx', 'attr', None, ([list], [None]))
+        task.Task.check_attr('xxx', 'attr', None, ([list], [None]))
 
     def testFailType(self):
-        pytest.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
+        pytest.raises(task.InvalidTask, task.Task.check_attr, 'xxx',
                       'attr', int, ([list], [False]))
 
     def testFailValue(self):
-        pytest.raises(task.InvalidTask, task.Task.check_attr_input, 'xxx',
+        pytest.raises(task.InvalidTask, task.Task.check_attr, 'xxx',
                       'attr', True, ([list], [False]))
 
 
@@ -58,6 +58,7 @@ class TestTaskInit(object):
         p1 = {'name':'p1', 'default':'p1-default'}
         p2 = {'name':'p2', 'default':'', 'short':'m'}
         t = task.Task("MyName", None, params=[p1, p2])
+        t._init_options()
         assert 'p1-default' == t.options['p1']
         assert '' == t.options['p2']
 
@@ -138,7 +139,7 @@ class TestTaskDeps(object):
 
     def test_calc_dep(self):
         my_task = task.Task("Task X", ["taskcmd"], calc_dep=["123"])
-        assert ["123"] == my_task.calc_dep
+        assert set(["123"]) == my_task.calc_dep
         assert ["123"] == my_task.calc_dep_stack
 
     def test_update_deps(self):
@@ -153,7 +154,7 @@ class TestTaskDeps(object):
                              })
         assert set(['fileX', 'fileY']) == my_task.file_dep
         assert ['resultX', 'taskY', 'resultY'] == my_task.task_dep
-        assert ['calcX', 'calcY'] == my_task.calc_dep
+        assert set(['calcX', 'calcY']) == my_task.calc_dep
         assert ['resultX', 'resultY'] == my_task.result_dep
         assert [None, True] == my_task.uptodate
 
@@ -162,8 +163,8 @@ class TestTask_Getargs(object):
     def test_ok(self):
         getargs = {'x' : 't1.x', 'y': 't2.z'}
         t = task.Task('t3', None, getargs=getargs)
-        assert 't1' in t.task_dep
-        assert 't2' in t.task_dep
+        assert 't1' in t.setup_tasks
+        assert 't2' in t.setup_tasks
 
     def test_invalid_desc(self):
         getargs = {'x' : 't1'}
@@ -173,7 +174,7 @@ class TestTask_Getargs(object):
     def test_many_dots(self):
         getargs = {'x': 't2:file.ext.x'}
         t = task.Task('t1', None, getargs=getargs)
-        assert 't2:file.ext' in t.task_dep
+        assert 't2:file.ext' in t.setup_tasks
 
 
 class TestTaskTitle(object):
