@@ -6,6 +6,7 @@ import pytest
 from doit import tools
 from doit import task
 
+
 def test_create_folder():
     def rm_dir():
         if os.path.exists(DIR_DEP):
@@ -37,6 +38,34 @@ class TestRunOnce(object):
         assert True == tools.run_once(t, t.values)
 
 
+class TestConfigChanged(object):\
+
+    def test_invalid_type(self):
+        class NotValid(object):pass
+        uptodate = tools.config_changed(NotValid())
+        pytest.raises(Exception, uptodate, None, None)
+
+    def test_string(self):
+        ua = tools.config_changed('a')
+        ub = tools.config_changed('b')
+        t1 = task.Task("TaskX", None, uptodate=[ua])
+        assert False == ua(t1, t1.values)
+        assert False == ub(t1, t1.values)
+        t1.execute()
+        assert True == ua(t1, t1.values)
+        assert False == ub(t1, t1.values)
+
+    def test_dict(self):
+        ua = tools.config_changed({'x':'a', 'y':1})
+        ub = tools.config_changed({'x':'b', 'y':1})
+        t1 = task.Task("TaskX", None, uptodate=[ua])
+        assert False == ua(t1, t1.values)
+        assert False == ub(t1, t1.values)
+        t1.execute()
+        assert True == ua(t1, t1.values)
+        assert False == ub(t1, t1.values)
+
+
 class TestTimeout(object):
     def test_invalid(self):
         pytest.raises(Exception, tools.timeout, "abc")
